"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var ethers_1 = require("ethers");
var licenseContractABI = [
    "function balanceOf(address account, uint256 id) view returns (uint256)",
    "function getPrice(address _token, uint256 _projectID) view returns (uint256)",
    "function getPrice(uint256 _projectID) view returns (uint256)",
    "function purchase(uint256 _projectID, address _recipient) payable",
    "function purchase(address _token, uint256 _projectID, address _recipient)"
];
var erc20ABI = [
    "function balanceOf(address account) view returns (uint256)",
    "function approve(address spender, uint256 amount) returns (bool)",
    "function allowance(address owner, address spender) view returns (uint256)"
];
var supportedChainIds = [80001, 137];
var contractAddresses = {
    80001: "0x3cE643dc61bb40bB0557316539f4A93016051b81",
    137: "0x3cE643dc61bb40bB0557316539f4A93016051b81",
};
var LicenseClient = /** @class */ (function () {
    /**
   * @constructor
   * Intializes the LicenseClient
   * @param {Provider} provider - instance of ethers.providers.Web3Provider or ethers.providers.JsonRpcProvider
   * @param {number} chainId - chainId of the contract to interact with
   * @example
   * const provider = new ethers.providers.Web3Provider(window.ethereum);
   * const licenseClient = new LicenseClient(provider, 80001);
   * @throws {Error} if provider and/or chainId is not provided
   * @throws {Error} if provided chainId is not supported
   */
    function LicenseClient(provider, chainId) {
        if (!provider || !chainId)
            throw new Error("Valist License SDK: Provider and chainId are required!");
        if (!supportedChainIds.includes(chainId))
            throw new Error("Valist License SDK: ChainId is not supported. Supported chainIds are 137, 80001");
        var licenseContractAddress = contractAddresses[chainId];
        this.provider = provider;
        this.chainId = chainId;
        this.signer = provider.getSigner();
        this.licenseClient = new ethers_1.Contract(licenseContractAddress, licenseContractABI, this.signer);
        console.log("Valist License SDK: initialized with chainId", chainId);
    }
    /**
   * This function checks if user has purchased the license
   * @param {ethers.BigNumberish} projectId - ID of the project
   * @param {string} signingMessage - message to be signed by user
   * @example const hasPurchased = await licenseClient.checkLicense(12, "I am signing this message");
   * @throws {Error} if connected provider chainId does not match with provided chainId
   * @returns {boolean} - true if user has purchased the license
   */
    LicenseClient.prototype.checkLicense = function (projectId, signingMessage) {
        if (signingMessage === void 0) { signingMessage = "I am signing this messag to authenticate"; }
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var chainId, signerAddress, signature, messgeHash, digest, recoveredAddress, balance;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.provider.getNetwork()];
                    case 1:
                        chainId = (_a.sent()).chainId;
                        if (chainId !== this.chainId)
                            throw new Error("Valist License SDK: Provider is connected to a different chainId than one specified in the constructor");
                        return [4 /*yield*/, this.signer.getAddress()];
                    case 2:
                        signerAddress = _a.sent();
                        return [4 /*yield*/, this.signer.signMessage(signingMessage)];
                    case 3:
                        signature = _a.sent();
                        messgeHash = ethers_1.utils.hashMessage(signingMessage);
                        digest = ethers_1.utils.arrayify(messgeHash);
                        recoveredAddress = ethers_1.utils.recoverAddress(digest, signature);
                        if (signerAddress.toLowerCase() !== recoveredAddress.toLowerCase())
                            throw new Error("Valist License SDK: Invalid signature");
                        return [4 /*yield*/, this.licenseClient.balanceOf(signerAddress, projectId)];
                    case 4:
                        balance = _a.sent();
                        return [2 /*return*/, balance.gt(0)];
                }
            });
        });
    };
    /**
     * Purchase license with native matic token
     * @param {ethers.BigNumberish} projectId - ID of the project
     * @param {string} recipient - address of the recipient
     * @example const tx = await licenseClient.purchaseLicense(12, "0xc49a...");
     * @throws {Error} if connected provider chainId does not match with provided chainId
     * @throws {Error} if price of the license is 0 which means project license may not exists or sold out
     * @throws {Error} if user does not have enough balance to purchase the license
     * @returns {Promise<ethers.ContractTransaction>} - instance of ethers.ContractTransaction
     */
    LicenseClient.prototype.purchaseLicense = function (projectId, recipient) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var chainId, price, balance, tranaction;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.provider.getNetwork()];
                    case 1:
                        chainId = (_a.sent()).chainId;
                        if (chainId !== this.chainId)
                            throw new Error("Valist License SDK: Provider is connected to a different chainId than one specified in the constructor");
                        return [4 /*yield*/, this.licenseClient["getPrice(uint256)"](projectId)];
                    case 2:
                        price = _a.sent();
                        if (price.eq(0))
                            throw new Error("Valist License SDK: License may not exists or sold out for this project");
                        return [4 /*yield*/, this.signer.getBalance()];
                    case 3:
                        balance = _a.sent();
                        if (balance.lt(price))
                            throw new Error("Valist License SDK: Insufficient balance to purchase license");
                        return [4 /*yield*/, this.licenseClient["purchase(uint256,address)"](projectId, recipient, { value: price })];
                    case 4:
                        tranaction = _a.sent();
                        return [2 /*return*/, tranaction];
                }
            });
        });
    };
    /**
     * Purchase license with any supported token
     * @param {ethers.BigNumberish} projectId - ID of the project
     * @param {string} recipient - address of the recipient
     * @param {string} tokenAddress - address of the token contract to be used for purchase
     * @example const tx = await licenseClient.purchaseLicense(12, "0xc49a...", "0x7d1a...");
     * @throws {Error} if connected provider chainId does not match with provided chainId
     * @throws {Error} if price of the license is 0 which means project license may not exists or sold out
     * @throws {Error} if token is not approved
     * @throws {Error} if token balance is less than price
     * @throws {Error} if token transfer fails
     * @returns {Promise<ethers.ContractTransaction>} - instance of ethers.ContractTransaction
     */
    LicenseClient.prototype.purchaseLicenseWithToken = function (projectId, recipient, tokenAddress) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var chainId, price, tokenContract, tokenBalance, _a, _b, tokenAllowance, approveTx, tranaction;
            return tslib_1.__generator(this, function (_c) {
                switch (_c.label) {
                    case 0: return [4 /*yield*/, this.provider.getNetwork()];
                    case 1:
                        chainId = (_c.sent()).chainId;
                        if (chainId !== this.chainId)
                            throw new Error("Valist License SDK: Provider is connected to a different chainId than one specified in the constructor");
                        return [4 /*yield*/, this.licenseClient["getPrice(address,uint256)"](tokenAddress, projectId)];
                    case 2:
                        price = _c.sent();
                        if (price.eq(0))
                            throw new Error("Valist License SDK: License may not exists or sold out for this project");
                        tokenContract = new ethers_1.Contract(tokenAddress, erc20ABI, this.signer);
                        _b = (_a = tokenContract).balanceOf;
                        return [4 /*yield*/, this.signer.getAddress()];
                    case 3: return [4 /*yield*/, _b.apply(_a, [_c.sent()])];
                    case 4:
                        tokenBalance = _c.sent();
                        if (tokenBalance.lt(price))
                            throw new Error("Valist License SDK: Insufficient token balance to purchase license");
                        return [4 /*yield*/, tokenContract.allowance(this.signer.getAddress(), this.licenseClient.address)];
                    case 5:
                        tokenAllowance = _c.sent();
                        if (!tokenAllowance.lt(price)) return [3 /*break*/, 8];
                        return [4 /*yield*/, tokenContract.approve(this.licenseClient.address, price.sub(tokenAllowance))];
                    case 6:
                        approveTx = _c.sent();
                        return [4 /*yield*/, approveTx.wait()];
                    case 7:
                        _c.sent();
                        _c.label = 8;
                    case 8: return [4 /*yield*/, this.licenseClient["purchase(address,uint256,address)"](tokenAddress, projectId, recipient, { value: price })];
                    case 9:
                        tranaction = _c.sent();
                        return [2 /*return*/, tranaction];
                }
            });
        });
    };
    return LicenseClient;
}());
exports.default = LicenseClient;
//# sourceMappingURL=index.js.map